@@ -11,6 +11,7 @@ print "1..6\n";
 our $base;
 
 my $prefix = "";
+my $ext = $ENV{PLAYTABTEST_EXT} || "ps";
 
 if ( -d "t" ) {
     $prefix = "t/";
@@ -45,3 +46,5 @@ sub differ {
     return 0 if $str1 eq $str2;
     1;
 }
+
+1;
